/** \file   my_io_dimacs.h
 *  \brief  IO functions for data files in DIMACS cnf format
 *  \author Takahisa Toda
 */
#ifndef MY_IO_DIMACS_H
#define MY_IO_DIMACS_H

#include <stdio.h>
#include "my_def.h"

#define DIMACS_NL      '0' //!< newline symbol in DIMACS cnf format
#define DIMACS_COMMENT 'c' //!< comment line in DIMACS cnf format
#define DIMACS_PROBLEM 'p' //!< problem line in DIMACS cnf format
#define DIMACS_SEPARATOR_TAB  '\t'
#define DIMACS_SEPARATOR_WS   ' '
// ASSUMPTION: the maximum number of characters in a line must be less than the following number.
#define MAX_LINESIZ  (UINTMAX_C(1) << 17)

/*---------------------------------------------------------------------------*/
/* Type declarations                                                         */
/*---------------------------------------------------------------------------*/

/** \brief an explicite data represention of a set family
 *  \note
 *  - Let sf point to this data structure.
 *  - sf->elem[i-1][j-1] is the j-th element in the i-th set. 
 *  - sf->elem[i-1][k] == INFTY if the i-th set contains exactly k elements.
 */
struct setfam {
  uintmax_t nsets;   //!< the number of sets with repetition (i.e., the number of rows in a data file)
  uintmax_t memsize; //!< the length of the array mem
  my_val **elem;
  my_val *mem;
};

/*---------------------------------------------------------------------------*/
/* Function prototypes                                                       */
/*---------------------------------------------------------------------------*/
extern int getfileinfo_dimacs(uintmax_t *pnrows, my_val *pmaxval, uintmax_t *ptotal_nent, FILE *in);
extern struct setfam *in2setfam_dimacs(uintmax_t nrows, uintmax_t total_nent, FILE *in);
extern void destroy_setfam(struct setfam *sf);
extern void format_eachrow(my_val *x[], uintmax_t nrows);
#endif /*MY_IO_DIMACS_H*/
